'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (flowtypes, filename) {
    var types = [];

    Object.keys(flowtypes).forEach(function (name) {
        if (flowtypes[name].length > 1) {
            flowtypes[name].forEach(function (flowtype, index) {
                types.push('export type ' + name + (index + 1) + ' = ' + objectType(flowtype) + ';');
            });
        } else {
            types.push('export type ' + name + ' = ' + objectType(flowtypes[name][0]) + ';');
        }
    });

    return '/* @flow\nDO NOT UPDATE THIS FILE MANUALLY\nAuto-generated flowtypes for Relay queries and fragments in ../' + filename + '\n*/\n\n' + types.join('\n\n') + '\n';
};

var INDENT = '    ';

;

function objectType(flowtype) {
    var indent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

    if (!Object.keys(flowtype).length) {
        return '{||}';
    }
    return '{|\n' + Object.keys(flowtype).map(function (name) {
        return '' + indent + INDENT + name + ': ' + fieldType(flowtype[name], indent);
    }).join(',\n') + '\n' + indent + '|}';
}

function fieldType(field, indent) {
    var flowType;

    switch (field.type) {
        case 'list':
            if (field.ofType.type === 'object' && !Object.keys(field.ofType.object).length) {
                // If there are no selections we want an opaque object even if it is a list
                flowType = '{||}';
            } else {
                flowType = 'Array<' + fieldType(field.ofType, indent) + '>';
            }
            break;
        case 'object':
            flowType = objectType(field.object, '' + indent + INDENT);
            break;
        default:
            flowType = field.type;
            break;
    }

    if (field.nonNull) {
        return flowType;
    }
    return 'null | ' + flowType;
}